@@ -3,7 +3,7 @@
 
 # "Tracking Bugs" - a chapter of "The Debugging Book"
 # Web site: https://www.debuggingbook.org/html/Tracking.html
-# Last change: 2021-03-06 12:41:30+01:00
+# Last change: 2021-05-26 17:47:15+02:00
 #
 # Copyright (c) 2021 CISPA Helmholtz Center for Information Security
 # Copyright (c) 2018-2020 Saarland University, authors, and contributors
@@ -76,6 +76,13 @@ if __name__ == '__main__':
 from . import Intro_Debugging
 
 import os
+import sys
+
+if __name__ == '__main__':
+    if 'CI' in os.environ:
+        # Can't run this in our continuous environment,
+        # since it can't run a headless Web browser
+        sys.exit(0)
 
 if __name__ == '__main__':
     assert os.getenv('USER') == 'zeller'
@@ -293,7 +300,7 @@ import time
 
 from multiprocessing import Process
 
-from typing import Tuple, Any
+from typing import Tuple
 
 def run_redmine(port: int) -> None:
     with_ruby(f'exec rails s -e production -p {port} > redmine.log 2>&1',
@@ -965,11 +972,3 @@ if __name__ == '__main__':
     print('\n## Background')
 
 
-
-## Exercises
-## ---------
-
-if __name__ == '__main__':
-    print('\n## Exercises')
-
-
